Index: sys/arm/broadcom/bcm2835/bcm2835_gpio.c
===================================================================
--- sys/arm/broadcom/bcm2835/bcm2835_gpio.c	(revision 254251)
+++ sys/arm/broadcom/bcm2835/bcm2835_gpio.c	(working copy)
@@ -52,6 +52,8 @@
 #include <dev/ofw/ofw_bus.h>
 #include <dev/ofw/ofw_bus_subr.h>
 
+#include <arm/broadcom/bcm2835/bcm2835_gpio.h>
+
 #include "gpio_if.h"
 
 #undef	DEBUG
@@ -87,17 +89,6 @@
 	struct bcm_gpio_sysctl	sc_sysctl[BCM_GPIO_PINS];
 };
 
-enum bcm_gpio_fsel {
-	BCM_GPIO_INPUT,
-	BCM_GPIO_OUTPUT,
-	BCM_GPIO_ALT5,
-	BCM_GPIO_ALT4,
-	BCM_GPIO_ALT0,
-	BCM_GPIO_ALT1,
-	BCM_GPIO_ALT2,
-	BCM_GPIO_ALT3,
-};
-
 enum bcm_gpio_pud {
 	BCM_GPIO_NONE,
 	BCM_GPIO_PULLDOWN,
@@ -257,6 +248,32 @@
 	BCM_GPIO_WRITE(sc, BCM_GPIO_GPPUDCLK(bank), 0);
 }
 
+void
+bcm_gpio_set_alternate(device_t dev, uint32_t pin, uint32_t nfunc)
+{
+	struct bcm_gpio_softc *sc;
+	int i;
+
+	sc = device_get_softc(dev);
+	BCM_GPIO_LOCK(sc);
+
+	/* Disable pull-up or pull-down on pin. */
+	bcm_gpio_set_pud(sc, pin, BCM_GPIO_NONE);
+
+	/* And now set the pin function. */
+	bcm_gpio_set_function(sc, pin, nfunc);
+
+	/* Update the pin flags. */
+	for (i = 0; i < sc->sc_gpio_npins; i++) {
+		if (sc->sc_gpio_pins[i].gp_pin == pin)
+			break;
+	}
+	if (i < sc->sc_gpio_npins)
+		sc->sc_gpio_pins[i].gp_flags = bcm_gpio_func_flag(nfunc);
+
+        BCM_GPIO_UNLOCK(sc);
+}
+
 static void
 bcm_gpio_pin_configure(struct bcm_gpio_softc *sc, struct gpio_pin *pin,
     unsigned int flags)
@@ -535,7 +552,7 @@
 	struct bcm_gpio_softc *sc;
 	struct bcm_gpio_sysctl *sc_sysctl;
 	uint32_t nfunc;
-	int i, error;
+	int error;
 
 	sc_sysctl = arg1;
 	sc = sc_sysctl->sc;
@@ -552,24 +569,8 @@
 	if (bcm_gpio_str_func(buf, &nfunc) != 0)
 		return (EINVAL);
 
-	BCM_GPIO_LOCK(sc);
-
-	/* Disable pull-up or pull-down on pin. */
-	bcm_gpio_set_pud(sc, sc_sysctl->pin, BCM_GPIO_NONE);
-
-	/* And now set the pin function. */
-	bcm_gpio_set_function(sc, sc_sysctl->pin, nfunc);
-
-	/* Update the pin flags. */
-	for (i = 0; i < sc->sc_gpio_npins; i++) {
-		if (sc->sc_gpio_pins[i].gp_pin == sc_sysctl->pin)
-			break;
-	}
-	if (i < sc->sc_gpio_npins)
-		sc->sc_gpio_pins[i].gp_flags = bcm_gpio_func_flag(nfunc);
-
-	BCM_GPIO_UNLOCK(sc);
-
+	/* Update the pin alternate function. */
+	bcm_gpio_set_alternate(sc->sc_dev, sc_sysctl->pin, nfunc);
 	return (0);
 }
 
--- /dev/null	2013-08-17 14:22:00.000000000 -0300
+++ sys/arm/broadcom/bcm2835/bcm2835_gpio.h	2013-08-17 14:21:48.098471481 -0300
@@ -0,0 +1,44 @@
+/*
+ * Copyright (c) 2013 Oleksandr Tymoshenko <gonzo@bluezbox.com>
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
+ * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
+ * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+ * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+ * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+ * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
+ *
+ * $FreeBSD$
+ */
+
+#ifndef	_BCM2835_GPIO_H_
+#define	_BCM2835_GPIO_H_
+
+enum bcm_gpio_fsel {
+	BCM_GPIO_INPUT,
+	BCM_GPIO_OUTPUT,
+	BCM_GPIO_ALT5,
+	BCM_GPIO_ALT4,
+	BCM_GPIO_ALT0,
+	BCM_GPIO_ALT1,
+	BCM_GPIO_ALT2,
+	BCM_GPIO_ALT3,
+};
+
+void bcm_gpio_set_alternate(device_t, uint32_t, uint32_t);
+
+#endif	/* _BCM2835_GPIO_H_ */
